@@ -1,6 +1,6 @@
 import logging
 
-from django.contrib.postgres.fields import JSONField
+from django.db.models import JSONField
 from django.forms import model_to_dict
 
 from isc_common.auth.models.user_permission import User_permission
@@ -32,7 +32,7 @@ class Widgets_treesManager(AuditManager):
 
         id_widget = data.get('id_widget')
 
-        if id == None and mode == None and mode == None:
+        if id is None and mode is None and mode is None:
             res, create = super().update_or_create(**data)
             return res
 
