@@ -15,6 +15,7 @@ from isc_common.common.functions import Common
 from isc_common.http.DSRequest import RequestData, DSRequest
 from isc_common.http.RPCResponse import RPCResponse, RPCResponseConstant
 from isc_common.http.response import JsonResponse
+from isc_common.json import BytesToJson
 
 logger = logging.getLogger(__name__)
 
@@ -36,11 +37,12 @@ class Body:
 
     def __init__(self, body, session):
         self.session = session
-        if not isinstance(body, dict):
-            raise Exception('body must by dict')
-
+        if isinstance(body, dict):
+            self.body = body
+        else:
+            body = BytesToJson(body)
         self.body = BodyWrapper(**body)
-        if self.body.transaction != None:
+        if self.body.transaction is not None:
             operations = self.body.transaction.get('operations')
             if isinstance(operations, list):
                 for operation in operations:
@@ -50,25 +52,49 @@ class Body:
                         self.body.httpHeaders = httpHeaders
                         break
 
-        if self.body.httpHeaders != None:
+        if self.body.httpHeaders is not None:
             self.body.httpHeaders = Wrapper(**self.body.httpHeaders)
-        elif self.body.data != None and self.body.data.get('httpHeaders') != None:
+        elif self.body.data is not None and self.body.data.get('httpHeaders') is not None:
             self.body.httpHeaders = Wrapper(**self.body.data.get('httpHeaders'))
 
     @property
     def login(self):
         from isc_common.auth.models.user import User
-        if self.body.login != None:
+        if self.body.login is not None:
             return self.body.login
-        elif self.body.httpHeaders != None and self.body.httpHeaders.USER_ID:
+        elif self.body.httpHeaders is not None and self.body.httpHeaders.USER_ID:
             return User.objects.get(id=self.body.httpHeaders.USER_ID).username
         else:
-            if self.session != None and self.session._session != None:
-                username = self.session._session.get('ws_channel').split('_')[1]
-                return username
+            if self.session is not None and self.session._session is not None:
+                return self.session._session.get('ws_channel').split('_')[1]
+            raise Exception('user not enable type')
+
+    @property
+    def user(self):
+        from isc_common.auth.models.user import User
+        if self.body.login is not None:
+            try:
+                return User.objects.get(username=self.body.login)
+            except User.DoesNotExist:
+                raise Exception('user not enable type')
+        elif self.body.httpHeaders is not None and self.body.httpHeaders.USER_ID:
+            return User.objects.get(id=self.body.httpHeaders.USER_ID)
+        else:
+            if self.session is not None and self.session._session is not None:
+                try:
+                    ws_channel = self.session._session.get('ws_channel')
+                    if ws_channel is not None:
+                        return User.objects.get(username=self.session._session.get('ws_channel').split('_')[1])
+                    else:
+                        return None
+                except User.DoesNotExist:
+                    raise Exception('user not enable type')
+            else:
+                raise Exception('user not enable type')
 
             raise Exception('user not enable type')
 
+
 def JsonWSResponseWithException(printing=False, printing_res=False):
     try:
         class JE:
@@ -97,26 +123,26 @@ def JsonWSResponseWithException(printing=False, printing_res=False):
                 request = args[0]
 
                 self.port = request.GET.get('ws_port')
-                if self.port == None:
+                if self.port is None:
                     self.port = request.GET.get('port')
-                    if self.port == None:
+                    if self.port is None:
                         self.port = settings.WS_PORT
-                        if self.port == None:
+                        if self.port is None:
                             raise Exception_WS_Enabled(Common.arraund_error('Do not have ws_port'))
 
                 self.host = request.GET.get('ws_host')
-                if self.host == None:
+                if self.host is None:
                     self.host = request.GET.get('host')
-                    if self.host == None:
+                    if self.host is None:
                         self.host = settings.WS_HOST
-                        if self.host == None:
+                        if self.host is None:
                             raise Exception_WS_Enabled(Common.arraund_error('Do not have ws_host'))
 
                 self.channel = request.GET.get('ws_channel')
-                if self.channel == None:
+                if self.channel is None:
                     body = BytesToJson(request.body)
                     body = Body(body, request.session)
-                    if settings.WS_CHANNEL != None:
+                    if settings.WS_CHANNEL is not None:
                         self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''
 
                 try:
@@ -247,24 +273,24 @@ def JsonWSPostResponseWithException(printing=False, printing_res=False):
             request = args[0]
 
             self.port = request.POST.get('ws_port')
-            if self.port == None:
+            if self.port is None:
                 self.port = settings.WS_PORT
                 if not self.port:
                     raise Exception(Common.arraund_error('Do not have ws_port'))
 
             self.host = request.POST.get('ws_host')
-            if self.host == None:
+            if self.host is None:
                 self.host = settings.WS_HOST
                 if not self.host:
                     raise Exception(Common.arraund_error('Do not have ws_host'))
 
             self.channel = request.POST.get('ws_channel')
-            if self.channel == None:
+            if self.channel is None:
                 body = BytesToJson(request.body)
                 body = Body(body, request.session)
-                if settings.WS_CHANNEL != None:
+                if settings.WS_CHANNEL is not None:
                     self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''
-                    if self.channel == None:
+                    if self.channel is None:
                         raise Exception(Common.arraund_error('Do not have ws_channel'))
 
             try:
