@@ -28,10 +28,19 @@ class TreeAuditModelQuerySet(AuditQuerySet):
                         order_by_clause='',
                         distinct='',
                         sql_text=None,
-                        fields='*') -> RawQuerySet:
+                        fields='*',
+                        limit=None
+                        ) -> RawQuerySet:
         db_name = self.model._meta.db_table
 
-        if id != None:
+        if limit is not None:
+            if not isinstance(limit, int):
+                raise Exception(f'limit must be int.')
+            limit = f'limit {limit}'
+        else:
+            limit = ''
+
+        if id is not None:
             if isinstance(id, int):
                 id = tuple([id])
             elif isinstance(id, str):
@@ -39,14 +48,14 @@ class TreeAuditModelQuerySet(AuditQuerySet):
             elif not isinstance(id, tuple):
                 raise Exception(f'id must be list or int')
 
-        if sql_text != None:
+        if sql_text is not None:
             res = super().raw(sql_text)
-        elif id != None:
+        elif id is not None:
             sql_text = f'''WITH RECURSIVE r AS (
-                                SELECT {fields}, 1 AS level
+                                select s.* from (SELECT {fields}, 1 AS level
                                 FROM {db_name}
                                 WHERE {child_id if include_self else parent_id} IN %s
-                                      {"and " + where_clause.replace("where", "") if where_clause else ""}  
+                                      {"and " + where_clause.replace("where", "") if where_clause else ""} {limit}) as s  
     
                                 union all
     
@@ -64,7 +73,7 @@ class TreeAuditModelQuerySet(AuditQuerySet):
             sql_text = f'''WITH RECURSIVE r AS (
                                             SELECT *, 1 AS level
                                             FROM {db_name}
-                                            WHERE {child_id if include_self else parent_id} {"IS NULL" if child_parent_value == None else child_parent_value_str}
+                                            WHERE {child_id if include_self else parent_id} {"IS NULL" if child_parent_value is None else child_parent_value_str}
                                                   {"and " + where_clause.replace("where", "") if where_clause else ""}  
 
                                             union all
@@ -123,6 +132,7 @@ class TreeAuditModelQuerySet(AuditQuerySet):
     def get_descendants_count(self,
                               id='id',
                               limit=None,
+                              limit1=None,
                               offset=None,
                               child_id='child_id',
                               parent_id='parent_id',
@@ -131,7 +141,7 @@ class TreeAuditModelQuerySet(AuditQuerySet):
                               where_clause='',
                               distinct='',
                               sql_text=None,
-                              fields='*'
+                              fields='*',
                               ) -> int:
         return len(self.get_descendants(
             id=id,
@@ -144,7 +154,8 @@ class TreeAuditModelQuerySet(AuditQuerySet):
             where_clause=where_clause,
             distinct=distinct,
             sql_text=sql_text,
-            fields=fields
+            fields=fields,
+            limit=limit1
         ))
 
 
@@ -159,6 +170,7 @@ class TreeAuditModelManager(AuditManager):
     def get_descendants(self,
                         id=None,
                         limit=None,
+                        limit1=None,
                         offset=None,
                         child_id='child_id',
                         parent_id='parent_id',
@@ -181,7 +193,8 @@ class TreeAuditModelManager(AuditManager):
             where_clause=where_clause,
             order_by_clause=order_by_clause,
             distinct=distinct,
-            fields=fields, sql_text=sql_text
+            fields=fields, sql_text=sql_text,
+            limit=limit1
         )
 
     def get_parents(self,
@@ -212,6 +225,7 @@ class TreeAuditModelManager(AuditManager):
     def get_descendants_count(self,
                               id=None,
                               limit=None,
+                              limit1=None,
                               offset=None,
                               child_id='child_id',
                               parent_id='parent_id',
@@ -233,7 +247,8 @@ class TreeAuditModelManager(AuditManager):
             distinct=distinct,
             where_clause=where_clause,
             sql_text=sql_text,
-            fields=fields
+            fields=fields,
+            limit1=limit1
         )
 
     def get_queryset(self):
